<?php
/*
 * Pool.php
 *
 * part of pfSense (https://www.pfsense.org)
 * Copyright (c) 2013-2021 Rubicon Communications, LLC (Netgate)
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

namespace Netgate\Services\ZFS;

use Nette\Utils\Arrays;
use Nette\Utils\Strings;

final class Pool implements ZFSNode {
	private $pool = [];

	private $types = [];

	public function __construct($pool) {
		$this->pool = $pool;

		$this->_initVdevTypes();
	}

	public function getVdevTypes(...$types) {
		if (empty($this->types)) {
			$this->_initVdevs();
		}

		$types = (is_array($types[0])) ? $types[0] : $types;

		return array_values(array_filter($this->types, function ($type) use ($types) {
			return (empty($types) || in_array($fs->getType(), $types));
		}));
	}

	public function getProperty($key, $default = null) {
		return Arrays::Get($this->pool, $key, $default);
	}

	public function getHtmlClass(string $prefix = null) : string {
		return Strings::webalize("{$prefix}{$this->getId()}");
	}

	public function getParentHtmlClass(string $prefix = null) : string {
		return $this->getHtmlClass($prefix);
	}

	public function getParent() : ZFSNode {
		return $this;
	}

	public function getName() {
		return $this->getProperty('name');
	}

	public function getId() {
		return $this->getProperty('id');
	}

	public function getGuid() {
		return $this->getProperty('guid');
	}

	public function getHealth() {
		return $this->getProperty(['properties', 'health', 'value']);
	}

	private function _initVdevTypes() {
		$filtered_types = array_filter($this->getProperty('groups'), function ($vdevs) {
			return !empty($vdevs);
		});

		foreach ($filtered_types as $type => $vdevs) {
			$type = $this->_getNewVdevTypeObject($this, $type, $vdevs);

			array_push($this->types, $type);
		}
	}

	private function _getNewVdevTypeObject($parent, $type, $vdevs) {
		return (new VDevType($parent, $type, $vdevs));
	}
}

?>