<?php
/*
 * VDevType.php
 *
 * part of pfSense (https://www.pfsense.org)
 * Copyright (c) 2013-2021 Rubicon Communications, LLC (Netgate)
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

namespace Netgate\Services\ZFS;

use Nette\Utils\Strings;

final class VDevType implements ZFSNode {
	private $parent = null;

	private $type = null;

	private $vdevs = [];

	public function __construct(Pool $parent, $type, $vdevs) {
		$this->parent = $parent;

		$this->type = $type;

		$this->vdevs = $vdevs;

		$this->_initVdevs();
	}

	public function getType() {
		return $this->type;
	}

	public function getHtmlClass(string $prefix = null) : string {
		return Strings::webalize("{$this->getParent()->getHtmlClass($prefix)}-{$this->getType()}");
	}

	public function getParentHtmlClass(string $prefix = null) : string {
		return Strings::webalize($this->getParent()->getHtmlClass($prefix));
	}

	public function getParent() : ZFSNode {
		return $this->parent;
	}

	public function getVdevs() {
		if (empty($this->vdevs)) {
			$this->_initVdevs();
		}

		return $this->vdevs;
	}

	public function getFlattenedVdevs() {
		$flattened = [];

		foreach($this->getVdevs() as $vdev) {
			$flattened = array_merge($flattened, $vdev->getChildVdevsAndSelf());
		}

		return $flattened;
	}

	private function _initVdevs() {
		$this->vdevs = array_map(function($vdev) {
			return $this->_getNewVdevObject($this, $vdev);
		}, $this->vdevs);
	}

	private function _getNewVdevObject($parent, $guid) {
		return (new VDev($parent, $guid));
	}
}

?>